/*******************************************************************************
*   Author       -  Kiraly Tibor
*                   www.tkiraaly.hu
*   Date         -  2016.03.09.
*   Chip         -  Atmel AVR
*   Compiler     -  avr-gcc (WinAVR 20080610) 4.3.0
*
*   AVR macros
*
*******************************************************************************/

#ifndef tkiraaly_avr_macros
#define tkiraaly_avr_macros




#define U8                   unsigned char
#define U16                  unsigned int




#define KHZ                  *1000
#define MHZ                  *1000000




#define B_1(sfr,bit)         sfr|= ( 1 << ( bit))               // egy regiszter adott bitjenek 1-re allitas
#define B_0(sfr,bit)         sfr&= ~( 1 << ( bit))              // egy regiszter adott bitjenek 0-ra allitas
#define B_R(sfr,bit)         bit_is_set( sfr, bit)              // egy regiszter egy bitjenek olvasasa




#define NOP                  __asm__ __volatile__ ("nop" ::)    // egy ures utasitas beszurasa




#define IT_ENABLE            __asm__ __volatile__ ("sei" ::)    // megszakitas engedelyezese
#define IT_DISABLE           __asm__ __volatile__ ("cli" ::)    // megszakitas tiltasa




#define USART_UCSRC          0B10000000          // UCSRC kivalasztasa
#define USART_ASYNCRON       0                   // asszinkron mukodes     - UCSR_C
#define USART_SYNCRON        0B01000000          // szinkron mukodes       - UCSR_C
#define USART_RX_ENABLE      0B00010000          // vetel engedelyezese    - UCSR_B
#define USART_TX_ENABLE      0B00001000          // adas engedelyezese     - UCSR_B
#define USART_NO_PARITY      0                   // nincs paritas bit      - UCSR_C
#define USART_EVEN_PARITY    0B00100000          // paros paritas          - UCSR_C
#define USART_ODD_PARITY     0B00110000          // paratlan paritas       - UCSR_C
#define USART_1_STOPBIT      0                   // 1 stopbit              - UCSR_C
#define USART_2_STOPBITS     0B00001000          // 2 stopbit              - UCSR_C
#define USART_7_BIT          0B00000100          // 7 bit-es karakterhossz - UCSR_C
#define USART_8_BIT          0B00000110          // 8 bit-es karakterhossz - UCSR_C




#define OCR_TIMER( ido_usec, osztas)  ido_usec* FCPU/ osztas/ 1000000UL+ 1UL
#define OCR_CLOCK( f_hz, osztas)      F_CPU/ osztas/ f_hz- 1UL
#define OCR_WAVE( f_hz, osztas)       F_CPU/ 2UL/ osztas/ f_hz- 1UL




// Port A

#define PA0_OUTPUT           B_1( DDRA, 0)       // Port A 0. bit kimenetre allitasa    
#define PA1_OUTPUT           B_1( DDRA, 1)
#define PA2_OUTPUT           B_1( DDRA, 2)
#define PA3_OUTPUT           B_1( DDRA, 3)
#define PA4_OUTPUT           B_1( DDRA, 4)
#define PA5_OUTPUT           B_1( DDRA, 5)
#define PA6_OUTPUT           B_1( DDRA, 6)
#define PA7_OUTPUT           B_1( DDRA, 7)
#define PAU_OUTPUT           DDRA|= 0B11110000   // Port A felso fel byte (upper nible) kimenetre allitasa
#define PAL_OUTPUT           DDRA|= 0B00001111   // Port A also fel byte (lower nible) kimenetre allitasa
#define PAA_OUTPUT           DDRA=  0B11111111   // Port A osszes (all) bit kimenetre allitasa

#define PA0_INPUT            B_0( DDRA, 0)       // Port A 0. bit bemenetre allitasa   
#define PA1_INPUT            B_0( DDRA, 1)
#define PA2_INPUT            B_0( DDRA, 2)
#define PA3_INPUT            B_0( DDRA, 3)
#define PA4_INPUT            B_0( DDRA, 4)
#define PA5_INPUT            B_0( DDRA, 5)
#define PA6_INPUT            B_0( DDRA, 6)
#define PA7_INPUT            B_0( DDRA, 7)
#define PAU_INPUT            DDRA&= 0B00001111
#define PAL_INPUT            DDRA&= 0B11110000
#define PAA_INPUT            DDRA=  0

#define PA0_PULLUP           B_1( PORTA, 0)      // Port A 0. bit belso felhuzas bekapcsolasa   
#define PA1_PULLUP           B_1( PORTA, 1)          
#define PA2_PULLUP           B_1( PORTA, 2)          
#define PA3_PULLUP           B_1( PORTA, 3)          
#define PA4_PULLUP           B_1( PORTA, 4)          
#define PA5_PULLUP           B_1( PORTA, 5)          
#define PA6_PULLUP           B_1( PORTA, 6)          
#define PA7_PULLUP           B_1( PORTA, 7)          
#define PAU_PULLUP           PORTA|= 0B11110000
#define PAL_PULLUP           PORTA|= 0B00001111
#define PAA_PULLUP           PORTA=  0B11111111

#define PA0_1                B_1( PORTA, 0)      // Port A 0. bit 1-be allitasa   
#define PA1_1                B_1( PORTA, 1)          
#define PA2_1                B_1( PORTA, 2)          
#define PA3_1                B_1( PORTA, 3)          
#define PA4_1                B_1( PORTA, 4)          
#define PA5_1                B_1( PORTA, 5)          
#define PA6_1                B_1( PORTA, 6)          
#define PA7_1                B_1( PORTA, 7)          
#define PAU_1                PORTA|= 0B11110000
#define PAL_1                PORTA|= 0B00001111
#define PAA_1                PORTA=  0B11111111

#define PA0_0                B_0( PORTA, 0)      // Port A 0. bit 0-ba allitasa    
#define PA1_0                B_0( PORTA, 1)          
#define PA2_0                B_0( PORTA, 2)          
#define PA3_0                B_0( PORTA, 3)          
#define PA4_0                B_0( PORTA, 4)          
#define PA5_0                B_0( PORTA, 5)          
#define PA6_0                B_0( PORTA, 6)          
#define PA7_0                B_0( PORTA, 7)          
#define PAU_0                PORTA&= 0B00001111
#define PAL_0                PORTA&= 0B11110000
#define PAA_0                PORTA=  0

#define PA0_R                B_R( PINA, 0)       // Port A 0. bit kiolvasasa   
#define PA1_R                B_R( PINA, 1)          
#define PA2_R                B_R( PINA, 2)          
#define PA3_R                B_R( PINA, 3)          
#define PA4_R                B_R( PINA, 4)          
#define PA5_R                B_R( PINA, 5)          
#define PA6_R                B_R( PINA, 6)          
#define PA7_R                B_R( PINA, 7)          
#define PAU_R                ( PINA & 0B11110000) >> 4
#define PAL_R                PINA & 0B00001111 
#define PAA_R                PINA 

#define PA0_W( X)            X ? B_1( PORTA, 0) : B_0( PORTA, 0)     // Port A 0. bit irasa 
#define PA1_W( X)            X ? B_1( PORTA, 1) : B_0( PORTA, 1)          
#define PA2_W( X)            X ? B_1( PORTA, 2) : B_0( PORTA, 2)          
#define PA3_W( X)            X ? B_1( PORTA, 3) : B_0( PORTA, 3)          
#define PA4_W( X)            X ? B_1( PORTA, 4) : B_0( PORTA, 4)          
#define PA5_W( X)            X ? B_1( PORTA, 5) : B_0( PORTA, 5)          
#define PA6_W( X)            X ? B_1( PORTA, 6) : B_0( PORTA, 6)          
#define PA7_W( X)            X ? B_1( PORTA, 7) : B_0( PORTA, 7)          
#define PAU_W( X)            PORTA= ( PORTA & 0B00001111) | ( ( X << 4) & 0B11110000)
#define PAL_W( X)            PORTA= ( PORTA & 0B11110000) | ( X & 0B00001111)
#define PAA_W( X)            PORTA= X          

// Port B

#define PB0_OUTPUT           B_1( DDRB, 0)          
#define PB1_OUTPUT           B_1( DDRB, 1)
#define PB2_OUTPUT           B_1( DDRB, 2)
#define PB3_OUTPUT           B_1( DDRB, 3)
#define PB4_OUTPUT           B_1( DDRB, 4)
#define PB5_OUTPUT           B_1( DDRB, 5)
#define PB6_OUTPUT           B_1( DDRB, 6)
#define PB7_OUTPUT           B_1( DDRB, 7)
#define PBU_OUTPUT           DDRB|= 0B11110000
#define PBL_OUTPUT           DDRB|= 0B00001111
#define PBA_OUTPUT           DDRB=  0B11111111

#define PB0_INPUT            B_0( DDRB, 0)          
#define PB1_INPUT            B_0( DDRB, 1)
#define PB2_INPUT            B_0( DDRB, 2)
#define PB3_INPUT            B_0( DDRB, 3)
#define PB4_INPUT            B_0( DDRB, 4)
#define PB5_INPUT            B_0( DDRB, 5)
#define PB6_INPUT            B_0( DDRB, 6)
#define PB7_INPUT            B_0( DDRB, 7)
#define PBU_INPUT            DDRB&= 0B00001111
#define PBL_INPUT            DDRB&= 0B11110000
#define PBA_INPUT            DDRB=  0

#define PB0_PULLUP           B_1( PORTB, 0)          
#define PB1_PULLUP           B_1( PORTB, 1)          
#define PB2_PULLUP           B_1( PORTB, 2)          
#define PB3_PULLUP           B_1( PORTB, 3)          
#define PB4_PULLUP           B_1( PORTB, 4)          
#define PB5_PULLUP           B_1( PORTB, 5)          
#define PB6_PULLUP           B_1( PORTB, 6)          
#define PB7_PULLUP           B_1( PORTB, 7)          
#define PBU_PULLUP           PORTB|= 0B11110000
#define PBL_PULLUP           PORTB|= 0B00001111
#define PBA_PULLUP           PORTB=  0B11111111

#define PB0_1                B_1( PORTB, 0)          
#define PB1_1                B_1( PORTB, 1)          
#define PB2_1                B_1( PORTB, 2)          
#define PB3_1                B_1( PORTB, 3)          
#define PB4_1                B_1( PORTB, 4)          
#define PB5_1                B_1( PORTB, 5)          
#define PB6_1                B_1( PORTB, 6)          
#define PB7_1                B_1( PORTB, 7)          
#define PBU_1                PORTB|= 0B11110000
#define PBL_1                PORTB|= 0B00001111
#define PBA_1                PORTB=  0B11111111

#define PB0_0                B_0( PORTB, 0)          
#define PB1_0                B_0( PORTB, 1)          
#define PB2_0                B_0( PORTB, 2)          
#define PB3_0                B_0( PORTB, 3)          
#define PB4_0                B_0( PORTB, 4)          
#define PB5_0                B_0( PORTB, 5)          
#define PB6_0                B_0( PORTB, 6)          
#define PB7_0                B_0( PORTB, 7)          
#define PBU_0                PORTB&= 0B00001111
#define PBL_0                PORTB&= 0B11110000
#define PBA_0                PORTB=  0

#define PB0_R                B_R( PINB, 0)          
#define PB1_R                B_R( PINB, 1)          
#define PB2_R                B_R( PINB, 2)          
#define PB3_R                B_R( PINB, 3)          
#define PB4_R                B_R( PINB, 4)          
#define PB5_R                B_R( PINB, 5)          
#define PB6_R                B_R( PINB, 6)          
#define PB7_R                B_R( PINB, 7)          
#define PBU_R                ( PINB & 0B11110000) >> 4
#define PBL_R                PINB & 0B00001111 
#define PBA_R                PINB 

#define PB0_W( X)            X ? B_1( PORTB, 0) : B_0( PORTB, 0)          
#define PB1_W( X)            X ? B_1( PORTB, 1) : B_0( PORTB, 1)          
#define PB2_W( X)            X ? B_1( PORTB, 2) : B_0( PORTB, 2)          
#define PB3_W( X)            X ? B_1( PORTB, 3) : B_0( PORTB, 3)          
#define PB4_W( X)            X ? B_1( PORTB, 4) : B_0( PORTB, 4)          
#define PB5_W( X)            X ? B_1( PORTB, 5) : B_0( PORTB, 5)          
#define PB6_W( X)            X ? B_1( PORTB, 6) : B_0( PORTB, 6)          
#define PB7_W( X)            X ? B_1( PORTB, 7) : B_0( PORTB, 7)          
#define PBU_W( X)            PORTB= ( PORTB & 0B00001111) | ( ( X << 4) & 0B11110000)
#define PBL_W( X)            PORTB= ( PORTB & 0B11110000) | ( X & 0B00001111)
#define PBA_W( X)            PORTB= X          

// Port C

#define PC0_OUTPUT           B_1( DDRC, 0)          
#define PC1_OUTPUT           B_1( DDRC, 1)
#define PC2_OUTPUT           B_1( DDRC, 2)
#define PC3_OUTPUT           B_1( DDRC, 3)
#define PC4_OUTPUT           B_1( DDRC, 4)
#define PC5_OUTPUT           B_1( DDRC, 5)
#define PC6_OUTPUT           B_1( DDRC, 6)
#define PC7_OUTPUT           B_1( DDRC, 7)
#define PCU_OUTPUT           DDRC|= 0B11110000
#define PCL_OUTPUT           DDRC|= 0B00001111
#define PCA_OUTPUT           DDRC=  0B11111111

#define PC0_INPUT            B_0( DDRC, 0)          
#define PC1_INPUT            B_0( DDRC, 1)
#define PC2_INPUT            B_0( DDRC, 2)
#define PC3_INPUT            B_0( DDRC, 3)
#define PC4_INPUT            B_0( DDRC, 4)
#define PC5_INPUT            B_0( DDRC, 5)
#define PC6_INPUT            B_0( DDRC, 6)
#define PC7_INPUT            B_0( DDRC, 7)
#define PCU_INPUT            DDRC&= 0B00001111
#define PCL_INPUT            DDRC&= 0B11110000
#define PCA_INPUT            DDRC=  0

#define PC0_PULLUP           B_1( PORTC, 0)          
#define PC1_PULLUP           B_1( PORTC, 1)          
#define PC2_PULLUP           B_1( PORTC, 2)          
#define PC3_PULLUP           B_1( PORTC, 3)          
#define PC4_PULLUP           B_1( PORTC, 4)          
#define PC5_PULLUP           B_1( PORTC, 5)          
#define PC6_PULLUP           B_1( PORTC, 6)          
#define PC7_PULLUP           B_1( PORTC, 7)          
#define PCU_PULLUP           PORTC|= 0B11110000
#define PCL_PULLUP           PORTC|= 0B00001111
#define PCA_PULLUP           PORTC=  0B11111111

#define PC0_1                B_1( PORTC, 0)          
#define PC1_1                B_1( PORTC, 1)          
#define PC2_1                B_1( PORTC, 2)          
#define PC3_1                B_1( PORTC, 3)          
#define PC4_1                B_1( PORTC, 4)          
#define PC5_1                B_1( PORTC, 5)          
#define PC6_1                B_1( PORTC, 6)          
#define PC7_1                B_1( PORTC, 7)          
#define PCU_1                PORTC|= 0B11110000
#define PCL_1                PORTC|= 0B00001111
#define PCA_1                PORTC=  0B11111111

#define PC0_0                B_0( PORTC, 0)          
#define PC1_0                B_0( PORTC, 1)          
#define PC2_0                B_0( PORTC, 2)          
#define PC3_0                B_0( PORTC, 3)          
#define PC4_0                B_0( PORTC, 4)          
#define PC5_0                B_0( PORTC, 5)          
#define PC6_0                B_0( PORTC, 6)          
#define PC7_0                B_0( PORTC, 7)          
#define PCU_0                PORTC&= 0B00001111
#define PCL_0                PORTC&= 0B11110000
#define PCA_0                PORTC=  0

#define PC0_R                B_R( PINC, 0)          
#define PC1_R                B_R( PINC, 1)          
#define PC2_R                B_R( PINC, 2)          
#define PC3_R                B_R( PINC, 3)          
#define PC4_R                B_R( PINC, 4)          
#define PC5_R                B_R( PINC, 5)          
#define PC6_R                B_R( PINC, 6)          
#define PC7_R                B_R( PINC, 7)          
#define PCU_R                ( PINC & 0B11110000) >> 4
#define PCL_R                PINC & 0B00001111 
#define PCA_R                PINC 

#define PC0_W( X)            X ? B_1( PORTC, 0) : B_0( PORTC, 0)          
#define PC1_W( X)            X ? B_1( PORTC, 1) : B_0( PORTC, 1)          
#define PC2_W( X)            X ? B_1( PORTC, 2) : B_0( PORTC, 2)          
#define PC3_W( X)            X ? B_1( PORTC, 3) : B_0( PORTC, 3)          
#define PC4_W( X)            X ? B_1( PORTC, 4) : B_0( PORTC, 4)          
#define PC5_W( X)            X ? B_1( PORTC, 5) : B_0( PORTC, 5)          
#define PC6_W( X)            X ? B_1( PORTC, 6) : B_0( PORTC, 6)          
#define PC7_W( X)            X ? B_1( PORTC, 7) : B_0( PORTC, 7)          
#define PCU_W( X)            PORTC= ( PORTC & 0B00001111) | ( ( X << 4) & 0B11110000)
#define PCL_W( X)            PORTC= ( PORTC & 0B11110000) | ( X & 0B00001111)
#define PCA_W( X)            PORTC= X          

// Port D

#define PD0_OUTPUT           B_1( DDRD, 0)          
#define PD1_OUTPUT           B_1( DDRD, 1)
#define PD2_OUTPUT           B_1( DDRD, 2)
#define PD3_OUTPUT           B_1( DDRD, 3)
#define PD4_OUTPUT           B_1( DDRD, 4)
#define PD5_OUTPUT           B_1( DDRD, 5)
#define PD6_OUTPUT           B_1( DDRD, 6)
#define PD7_OUTPUT           B_1( DDRD, 7)
#define PDU_OUTPUT           DDRD|= 0B11110000
#define PDL_OUTPUT           DDRD|= 0B00001111
#define PDA_OUTPUT           DDRD=  0B11111111

#define PD0_INPUT            B_0( DDRD, 0)          
#define PD1_INPUT            B_0( DDRD, 1)
#define PD2_INPUT            B_0( DDRD, 2)
#define PD3_INPUT            B_0( DDRD, 3)
#define PD4_INPUT            B_0( DDRD, 4)
#define PD5_INPUT            B_0( DDRD, 5)
#define PD6_INPUT            B_0( DDRD, 6)
#define PD7_INPUT            B_0( DDRD, 7)
#define PDU_INPUT            DDRD&= 0B00001111
#define PDL_INPUT            DDRD&= 0B11110000
#define PDA_INPUT            DDRD=  0

#define PD0_PULLUP           B_1( PORTD, 0)          
#define PD1_PULLUP           B_1( PORTD, 1)          
#define PD2_PULLUP           B_1( PORTD, 2)          
#define PD3_PULLUP           B_1( PORTD, 3)          
#define PD4_PULLUP           B_1( PORTD, 4)          
#define PD5_PULLUP           B_1( PORTD, 5)          
#define PD6_PULLUP           B_1( PORTD, 6)          
#define PD7_PULLUP           B_1( PORTD, 7)          
#define PDU_PULLUP           PORTD|= 0B11110000
#define PDL_PULLUP           PORTD|= 0B00001111
#define PDA_PULLUP           PORTD=  0B11111111

#define PD0_1                B_1( PORTD, 0)          
#define PD1_1                B_1( PORTD, 1)          
#define PD2_1                B_1( PORTD, 2)          
#define PD3_1                B_1( PORTD, 3)          
#define PD4_1                B_1( PORTD, 4)          
#define PD5_1                B_1( PORTD, 5)          
#define PD6_1                B_1( PORTD, 6)          
#define PD7_1                B_1( PORTD, 7)          
#define PDU_1                PORTD|= 0B11110000
#define PDL_1                PORTD|= 0B00001111
#define PDA_1                PORTD=  0B11111111

#define PD0_0                B_0( PORTD, 0)          
#define PD1_0                B_0( PORTD, 1)          
#define PD2_0                B_0( PORTD, 2)          
#define PD3_0                B_0( PORTD, 3)          
#define PD4_0                B_0( PORTD, 4)          
#define PD5_0                B_0( PORTD, 5)          
#define PD6_0                B_0( PORTD, 6)          
#define PD7_0                B_0( PORTD, 7)          
#define PDU_0                PORTD&= 0B00001111
#define PDL_0                PORTD&= 0B11110000
#define PDA_0                PORTD=  0

#define PD0_R                B_R( PIND, 0)          
#define PD1_R                B_R( PIND, 1)          
#define PD2_R                B_R( PIND, 2)          
#define PD3_R                B_R( PIND, 3)          
#define PD4_R                B_R( PIND, 4)          
#define PD5_R                B_R( PIND, 5)          
#define PD6_R                B_R( PIND, 6)          
#define PD7_R                B_R( PIND, 7)          
#define PDU_R                ( PIND & 0B11110000) >> 4
#define PDL_R                PIND & 0B00001111 
#define PDA_R                PIND 

#define PD0_W( X)            X ? B_1( PORTD, 0) : B_0( PORTD, 0)          
#define PD1_W( X)            X ? B_1( PORTD, 1) : B_0( PORTD, 1)          
#define PD2_W( X)            X ? B_1( PORTD, 2) : B_0( PORTD, 2)          
#define PD3_W( X)            X ? B_1( PORTD, 3) : B_0( PORTD, 3)          
#define PD4_W( X)            X ? B_1( PORTD, 4) : B_0( PORTC, 4)          
#define PD5_W( X)            X ? B_1( PORTD, 5) : B_0( PORTD, 5)          
#define PD6_W( X)            X ? B_1( PORTD, 6) : B_0( PORTD, 6)          
#define PD7_W( X)            X ? B_1( PORTD, 7) : B_0( PORTD, 7)          
#define PDU_W( X)            PORTD= ( PORTD & 0B00001111) | ( ( X << 4) & 0B11110000)
#define PDL_W( X)            PORTD= ( PORTD & 0B11110000) | ( X & 0B00001111)
#define PDA_W( X)            PORTD= X          

// Port E

#define PPE0                 0x40
#define PPE1                 0x41
#define PPE2                 0x42
#define PPE3                 0x43
#define PPE4                 0x44
#define PPE5                 0x45
#define PPE6                 0x46
#define PPE7                 0x47
#define PPEU                 0x48
#define PPEL                 0x49
#define PPEA                 0x4A

#define PE0_OUTPUT           B_1( DDRE, 0)          
#define PE1_OUTPUT           B_1( DDRE, 1)
#define PE2_OUTPUT           B_1( DDRE, 2)
#define PE3_OUTPUT           B_1( DDRE, 3)
#define PE4_OUTPUT           B_1( DDRE, 4)
#define PE5_OUTPUT           B_1( DDRE, 5)
#define PE6_OUTPUT           B_1( DDRE, 6)
#define PE7_OUTPUT           B_1( DDRE, 7)
#define PEU_OUTPUT           DDRE|= 0B11110000
#define PEL_OUTPUT           DDRE|= 0B00001111
#define PEA_OUTPUT           DDRE=  0B11111111

#define PE0_INPUT            B_0( DDRE, 0)          
#define PE1_INPUT            B_0( DDRE, 1)
#define PE2_INPUT            B_0( DDRE, 2)
#define PE3_INPUT            B_0( DDRE, 3)
#define PE4_INPUT            B_0( DDRE, 4)
#define PE5_INPUT            B_0( DDRE, 5)
#define PE6_INPUT            B_0( DDRE, 6)
#define PE7_INPUT            B_0( DDRE, 7)
#define PEU_INPUT            DDRE&= 0B00001111
#define PEL_INPUT            DDRE&= 0B11110000
#define PEA_INPUT            DDRE=  0

#define PE0_PULLUP           B_1( PORTE, 0)          
#define PE1_PULLUP           B_1( PORTE, 1)          
#define PE2_PULLUP           B_1( PORTE, 2)          
#define PE3_PULLUP           B_1( PORTE, 3)          
#define PE4_PULLUP           B_1( PORTE, 4)          
#define PE5_PULLUP           B_1( PORTE, 5)          
#define PE6_PULLUP           B_1( PORTE, 6)          
#define PE7_PULLUP           B_1( PORTE, 7)          
#define PEU_PULLUP           PORTE|= 0B11110000
#define PEL_PULLUP           PORTE|= 0B00001111
#define PEA_PULLUP           PORTE=  0B11111111

#define PE0_1                B_1( PORTE, 0)          
#define PE1_1                B_1( PORTE, 1)          
#define PE2_1                B_1( PORTE, 2)          
#define PE3_1                B_1( PORTE, 3)          
#define PE4_1                B_1( PORTE, 4)          
#define PE5_1                B_1( PORTE, 5)          
#define PE6_1                B_1( PORTE, 6)          
#define PE7_1                B_1( PORTE, 7)          
#define PEU_1                PORTE|= 0B11110000
#define PEL_1                PORTE|= 0B00001111
#define PEA_1                PORTE=  0B11111111

#define PE0_0                B_0( PORTE, 0)          
#define PE1_0                B_0( PORTE, 1)          
#define PE2_0                B_0( PORTE, 2)          
#define PE3_0                B_0( PORTE, 3)          
#define PE4_0                B_0( PORTE, 4)          
#define PE5_0                B_0( PORTE, 5)          
#define PE6_0                B_0( PORTE, 6)          
#define PE7_0                B_0( PORTE, 7)          
#define PEU_0                PORTE&= 0B00001111
#define PEL_0                PORTE&= 0B11110000
#define PEA_0                PORTE=  0

#define PE0_R                B_R( PINE, 0)          
#define PE1_R                B_R( PINE, 1)          
#define PE2_R                B_R( PINE, 2)          
#define PE3_R                B_R( PINE, 3)          
#define PE4_R                B_R( PINE, 4)          
#define PE5_R                B_R( PINE, 5)          
#define PE6_R                B_R( PINE, 6)          
#define PE7_R                B_R( PINE, 7)          
#define PEU_R                ( PINE & 0B11110000) >> 4
#define PEL_R                PINE & 0B00001111 
#define PEA_R                PINE 

#define PE0_W( X)            X ? B_1( PORTE, 0) : B_0( PORTE, 0)          
#define PE1_W( X)            X ? B_1( PORTE, 1) : B_0( PORTE, 1)          
#define PE2_W( X)            X ? B_1( PORTE, 2) : B_0( PORTE, 2)          
#define PE3_W( X)            X ? B_1( PORTE, 3) : B_0( PORTE, 3)          
#define PE4_W( X)            X ? B_1( PORTE, 4) : B_0( PORTE, 4)          
#define PE5_W( X)            X ? B_1( PORTE, 5) : B_0( PORTE, 5)          
#define PE6_W( X)            X ? B_1( PORTE, 6) : B_0( PORTE, 6)          
#define PE7_W( X)            X ? B_1( PORTE, 7) : B_0( PORTE, 7)          
#define PEU_W( X)            PORTE= ( PORTE & 0B00001111) | ( ( X << 4) & 0B11110000)
#define PEL_W( X)            PORTE= ( PORTE & 0B11110000) | ( X & 0B00001111)
#define PEA_W( X)            PORTE= X          

// Port F

#define PF0_OUTPUT           B_1( DDRF, 0)          
#define PF1_OUTPUT           B_1( DDRF, 1)
#define PF2_OUTPUT           B_1( DDRF, 2)
#define PF3_OUTPUT           B_1( DDRF, 3)
#define PF4_OUTPUT           B_1( DDRF, 4)
#define PF5_OUTPUT           B_1( DDRF, 5)
#define PF6_OUTPUT           B_1( DDRF, 6)
#define PF7_OUTPUT           B_1( DDRF, 7)
#define PFU_OUTPUT           DDRF|= 0B11110000
#define PFL_OUTPUT           DDRF|= 0B00001111
#define PFA_OUTPUT           DDRF=  0B11111111

#define PF0_INPUT            B_0( DDRF, 0)          
#define PF1_INPUT            B_0( DDRF, 1)
#define PF2_INPUT            B_0( DDRF, 2)
#define PF3_INPUT            B_0( DDRF, 3)
#define PF4_INPUT            B_0( DDRF, 4)
#define PF5_INPUT            B_0( DDRF, 5)
#define PF6_INPUT            B_0( DDRF, 6)
#define PF7_INPUT            B_0( DDRF, 7)
#define PFU_INPUT            DDRF&= 0B00001111
#define PFL_INPUT            DDRF&= 0B11110000
#define PFA_INPUT            DDRF=  0

#define PF0_PULLUP           B_1( PORTF, 0)          
#define PF1_PULLUP           B_1( PORTF, 1)          
#define PF2_PULLUP           B_1( PORTF, 2)          
#define PF3_PULLUP           B_1( PORTF, 3)          
#define PF4_PULLUP           B_1( PORTF, 4)          
#define PF5_PULLUP           B_1( PORTF, 5)          
#define PF6_PULLUP           B_1( PORTF, 6)          
#define PF7_PULLUP           B_1( PORTF, 7)          
#define PFU_PULLUP           PORTF|= 0B11110000
#define PFL_PULLUP           PORTF|= 0B00001111
#define PFA_PULLUP           PORTF=  0B11111111

#define PF0_1                B_1( PORTF, 0)          
#define PF1_1                B_1( PORTF, 1)          
#define PF2_1                B_1( PORTF, 2)          
#define PF3_1                B_1( PORTF, 3)          
#define PF4_1                B_1( PORTF, 4)          
#define PF5_1                B_1( PORTF, 5)          
#define PF6_1                B_1( PORTF, 6)          
#define PF7_1                B_1( PORTF, 7)          
#define PFU_1                PORTF|= 0B11110000
#define PFL_1                PORTF|= 0B00001111
#define PFA_1                PORTF=  0B11111111

#define PF0_0                B_0( PORTF, 0)          
#define PF1_0                B_0( PORTF, 1)          
#define PF2_0                B_0( PORTF, 2)          
#define PF3_0                B_0( PORTF, 3)          
#define PF4_0                B_0( PORTF, 4)          
#define PF5_0                B_0( PORTF, 5)          
#define PF6_0                B_0( PORTF, 6)          
#define PF7_0                B_0( PORTF, 7)          
#define PFU_0                PORTF&= 0B00001111
#define PFL_0                PORTF&= 0B11110000
#define PFA_0                PORTF=  0

#define PF0_R                B_R( PINF, 0)          
#define PF1_R                B_R( PINF, 1)          
#define PF2_R                B_R( PINF, 2)          
#define PF3_R                B_R( PINF, 3)          
#define PF4_R                B_R( PINF, 4)          
#define PF5_R                B_R( PINF, 5)          
#define PF6_R                B_R( PINF, 6)          
#define PF7_R                B_R( PINF, 7)          
#define PFU_R                ( PINF & 0B11110000) >> 4
#define PFL_R                PINF & 0B00001111 
#define PFA_R                PINF 

#define PF0_W( X)            X ? B_1( PORTF, 0) : B_0( PORTF, 0)          
#define PF1_W( X)            X ? B_1( PORTF, 1) : B_0( PORTF, 1)          
#define PF2_W( X)            X ? B_1( PORTF, 2) : B_0( PORTF, 2)          
#define PF3_W( X)            X ? B_1( PORTF, 3) : B_0( PORTF, 3)          
#define PF4_W( X)            X ? B_1( PORTF, 4) : B_0( PORTF, 4)          
#define PF5_W( X)            X ? B_1( PORTF, 5) : B_0( PORTF, 5)          
#define PF6_W( X)            X ? B_1( PORTF, 6) : B_0( PORTF, 6)          
#define PF7_W( X)            X ? B_1( PORTF, 7) : B_0( PORTF, 7)          
#define PFU_W( X)            PORTF= ( PORTF & 0B00001111) | ( ( X << 4) & 0B11110000)
#define PFL_W( X)            PORTF= ( PORTF & 0B11110000) | ( X & 0B00001111)
#define PFA_W( X)            PORTF= X          

// Port G

#define PG0_OUTPUT           B_1( DDRG, 0)          
#define PG1_OUTPUT           B_1( DDRG, 1)
#define PG2_OUTPUT           B_1( DDRG, 2)
#define PG3_OUTPUT           B_1( DDRG, 3)
#define PG4_OUTPUT           B_1( DDRG, 4)
#define PG5_OUTPUT           B_1( DDRG, 5)
#define PG6_OUTPUT           B_1( DDRG, 6)
#define PG7_OUTPUT           B_1( DDRG, 7)
#define PGU_OUTPUT           DDRG|= 0B11110000
#define PGL_OUTPUT           DDRG|= 0B00001111
#define PGA_OUTPUT           DDRG=  0B11111111

#define PG0_INPUT            B_0( DDRG, 0)          
#define PG1_INPUT            B_0( DDRG, 1)
#define PG2_INPUT            B_0( DDRG, 2)
#define PG3_INPUT            B_0( DDRG, 3)
#define PG4_INPUT            B_0( DDRG, 4)
#define PG5_INPUT            B_0( DDRG, 5)
#define PG6_INPUT            B_0( DDRG, 6)
#define PG7_INPUT            B_0( DDRG, 7)
#define PGU_INPUT            DDRG&= 0B00001111
#define PGL_INPUT            DDRG&= 0B11110000
#define PGA_INPUT            DDRG=  0

#define PG0_PULLUP           B_1( PORTG, 0)          
#define PG1_PULLUP           B_1( PORTG, 1)          
#define PG2_PULLUP           B_1( PORTG, 2)          
#define PG3_PULLUP           B_1( PORTG, 3)          
#define PG4_PULLUP           B_1( PORTG, 4)          
#define PG5_PULLUP           B_1( PORTG, 5)          
#define PG6_PULLUP           B_1( PORTG, 6)          
#define PG7_PULLUP           B_1( PORTG, 7)          
#define PGU_PULLUP           PORTG|= 0B11110000
#define PGL_PULLUP           PORTG|= 0B00001111
#define PGA_PULLUP           PORTG=  0B11111111

#define PG0_1                B_1( PORTG, 0)          
#define PG1_1                B_1( PORTG, 1)          
#define PG2_1                B_1( PORTG, 2)          
#define PG3_1                B_1( PORTG, 3)          
#define PG4_1                B_1( PORTG, 4)          
#define PG5_1                B_1( PORTG, 5)          
#define PG6_1                B_1( PORTG, 6)          
#define PG7_1                B_1( PORTG, 7)          
#define PGU_1                PORTG|= 0B11110000
#define PGL_1                PORTG|= 0B00001111
#define PGA_1                PORTG=  0B11111111

#define PG0_0                B_0( PORTG, 0)          
#define PG1_0                B_0( PORTG, 1)          
#define PG2_0                B_0( PORTG, 2)          
#define PG3_0                B_0( PORTG, 3)          
#define PG4_0                B_0( PORTG, 4)          
#define PG5_0                B_0( PORTG, 5)          
#define PG6_0                B_0( PORTG, 6)          
#define PG7_0                B_0( PORTG, 7)          
#define PGU_0                PORTG&= 0B00001111
#define PGL_0                PORTG&= 0B11110000
#define PGA_0                PORTG=  0

#define PG0_R                B_R( PING, 0)          
#define PG1_R                B_R( PING, 1)          
#define PG2_R                B_R( PING, 2)          
#define PG3_R                B_R( PING, 3)          
#define PG4_R                B_R( PING, 4)          
#define PG5_R                B_R( PING, 5)          
#define PG6_R                B_R( PING, 6)          
#define PG7_R                B_R( PING, 7)          
#define PGU_R                ( PING & 0B11110000) >> 4
#define PGL_R                PING & 0B00001111 
#define PGA_R                PING 

#define PG0_W( X)            X ? B_1( PORTG, 0) : B_0( PORTG, 0)          
#define PG1_W( X)            X ? B_1( PORTG, 1) : B_0( PORTG, 1)          
#define PG2_W( X)            X ? B_1( PORTG, 2) : B_0( PORTG, 2)          
#define PG3_W( X)            X ? B_1( PORTG, 3) : B_0( PORTG, 3)          
#define PG4_W( X)            X ? B_1( PORTG, 4) : B_0( PORTG, 4)          
#define PG5_W( X)            X ? B_1( PORTG, 5) : B_0( PORTG, 5)          
#define PG6_W( X)            X ? B_1( PORTG, 6) : B_0( PORTG, 6)          
#define PG7_W( X)            X ? B_1( PORTG, 7) : B_0( PORTG, 7)          
#define PGU_W( X)            PORTG= ( PORTG & 0B00001111) | ( ( X << 4) & 0B11110000)
#define PGL_W( X)            PORTG= ( PORTG & 0B11110000) | ( X & 0B00001111)
#define PGA_W( X)            PORTG= X          

// Port H

#define PH0_OUTPUT           B_1( DDRH, 0)          
#define PH1_OUTPUT           B_1( DDRH, 1)
#define PH2_OUTPUT           B_1( DDRH, 2)
#define PH3_OUTPUT           B_1( DDRH, 3)
#define PH4_OUTPUT           B_1( DDRH, 4)
#define PH5_OUTPUT           B_1( DDRH, 5)
#define PH6_OUTPUT           B_1( DDRH, 6)
#define PH7_OUTPUT           B_1( DDRH, 7)
#define PHU_OUTPUT           DDRH|= 0B11110000
#define PHL_OUTPUT           DDRH|= 0B00001111
#define PHA_OUTPUT           DDRH=  0B11111111

#define PH0_INPUT            B_0( DDRH, 0)          
#define PH1_INPUT            B_0( DDRH, 1)
#define PH2_INPUT            B_0( DDRH, 2)
#define PH3_INPUT            B_0( DDRH, 3)
#define PH4_INPUT            B_0( DDRH, 4)
#define PH5_INPUT            B_0( DDRH, 5)
#define PH6_INPUT            B_0( DDRH, 6)
#define PH7_INPUT            B_0( DDRH, 7)
#define PHU_INPUT            DDRH&= 0B00001111
#define PHL_INPUT            DDRH&= 0B11110000
#define PHA_INPUT            DDRH=  0

#define PH0_PULLUP           B_1( PORTH, 0)          
#define PH1_PULLUP           B_1( PORTH, 1)          
#define PH2_PULLUP           B_1( PORTH, 2)          
#define PH3_PULLUP           B_1( PORTH, 3)          
#define PH4_PULLUP           B_1( PORTH, 4)          
#define PH5_PULLUP           B_1( PORTH, 5)          
#define PH6_PULLUP           B_1( PORTH, 6)          
#define PH7_PULLUP           B_1( PORTH, 7)          
#define PHU_PULLUP           PORTH|= 0B11110000
#define PHL_PULLUP           PORTH|= 0B00001111
#define PHA_PULLUP           PORTH=  0B11111111

#define PH0_1                B_1( PORTH, 0)          
#define PH1_1                B_1( PORTH, 1)          
#define PH2_1                B_1( PORTH, 2)          
#define PH3_1                B_1( PORTH, 3)          
#define PH4_1                B_1( PORTH, 4)          
#define PH5_1                B_1( PORTH, 5)          
#define PH6_1                B_1( PORTH, 6)          
#define PH7_1                B_1( PORTH, 7)          
#define PHU_1                PORTH|= 0B11110000
#define PHL_1                PORTH|= 0B00001111
#define PHA_1                PORTH=  0B11111111

#define PH0_0                B_0( PORTH, 0)          
#define PH1_0                B_0( PORTH, 1)          
#define PH2_0                B_0( PORTH, 2)          
#define PH3_0                B_0( PORTH, 3)          
#define PH4_0                B_0( PORTH, 4)          
#define PH5_0                B_0( PORTH, 5)          
#define PH6_0                B_0( PORTH, 6)          
#define PH7_0                B_0( PORTH, 7)          
#define PHU_0                PORTH&= 0B00001111
#define PHL_0                PORTH&= 0B11110000
#define PHA_0                PORTH=  0

#define PH0_R                B_R( PINH, 0)          
#define PH1_R                B_R( PINH, 1)          
#define PH2_R                B_R( PINH, 2)          
#define PH3_R                B_R( PINH, 3)          
#define PH4_R                B_R( PINH, 4)          
#define PH5_R                B_R( PINH, 5)          
#define PH6_R                B_R( PINH, 6)          
#define PH7_R                B_R( PINH, 7)          
#define PHU_R                ( PINH & 0B11110000) >> 4
#define PHL_R                PINH & 0B00001111 
#define PHA_R                PINH 

#define PH0_W( X)            X ? B_1( PORTH, 0) : B_0( PORTH, 0)          
#define PH1_W( X)            X ? B_1( PORTH, 1) : B_0( PORTH, 1)          
#define PH2_W( X)            X ? B_1( PORTH, 2) : B_0( PORTH, 2)          
#define PH3_W( X)            X ? B_1( PORTH, 3) : B_0( PORTH, 3)          
#define PH4_W( X)            X ? B_1( PORTH, 4) : B_0( PORTH, 4)          
#define PH5_W( X)            X ? B_1( PORTH, 5) : B_0( PORTH, 5)          
#define PH6_W( X)            X ? B_1( PORTH, 6) : B_0( PORTH, 6)          
#define PH7_W( X)            X ? B_1( PORTH, 7) : B_0( PORTH, 7)          
#define PHU_W( X)            PORTH= ( PORTH & 0B00001111) | ( ( X << 4) & 0B11110000)
#define PHL_W( X)            PORTH= ( PORTH & 0B11110000) | ( X & 0B00001111)
#define PHA_W( X)            PORTH= X          

// Port I

#define PI0_OUTPUT           B_1( DDRI, 0)          
#define PI1_OUTPUT           B_1( DDRI, 1)
#define PI2_OUTPUT           B_1( DDRI, 2)
#define PI3_OUTPUT           B_1( DDRI, 3)
#define PI4_OUTPUT           B_1( DDRI, 4)
#define PI5_OUTPUT           B_1( DDRI, 5)
#define PI6_OUTPUT           B_1( DDRI, 6)
#define PI7_OUTPUT           B_1( DDRI, 7)
#define PIU_OUTPUT           DDRI|= 0B11110000
#define PIL_OUTPUT           DDRI|= 0B00001111
#define PIA_OUTPUT           DDRI=  0B11111111

#define PI0_INPUT            B_0( DDRI, 0)          
#define PI1_INPUT            B_0( DDRI, 1)
#define PI2_INPUT            B_0( DDRI, 2)
#define PI3_INPUT            B_0( DDRI, 3)
#define PI4_INPUT            B_0( DDRI, 4)
#define PI5_INPUT            B_0( DDRI, 5)
#define PI6_INPUT            B_0( DDRI, 6)
#define PI7_INPUT            B_0( DDRI, 7)
#define PIU_INPUT            DDRI&= 0B00001111
#define PIL_INPUT            DDRI&= 0B11110000
#define PIA_INPUT            DDRI=  0

#define PI0_PULLUP           B_1( PORTI, 0)          
#define PI1_PULLUP           B_1( PORTI, 1)          
#define PI2_PULLUP           B_1( PORTI, 2)          
#define PI3_PULLUP           B_1( PORTI, 3)          
#define PI4_PULLUP           B_1( PORTI, 4)          
#define PI5_PULLUP           B_1( PORTI, 5)          
#define PI6_PULLUP           B_1( PORTI, 6)          
#define PI7_PULLUP           B_1( PORTI, 7)          
#define PIU_PULLUP           PORTI|= 0B11110000
#define PIL_PULLUP           PORTI|= 0B00001111
#define PIA_PULLUP           PORTI=  0B11111111

#define PI0_1                B_1( PORTI, 0)          
#define PI1_1                B_1( PORTI, 1)          
#define PI2_1                B_1( PORTI, 2)          
#define PI3_1                B_1( PORTI, 3)          
#define PI4_1                B_1( PORTI, 4)          
#define PI5_1                B_1( PORTI, 5)          
#define PI6_1                B_1( PORTI, 6)          
#define PI7_1                B_1( PORTI, 7)          
#define PIU_1                PORTI|= 0B11110000
#define PIL_1                PORTI|= 0B00001111
#define PIA_1                PORTI=  0B11111111

#define PI0_0                B_0( PORTI, 0)          
#define PI1_0                B_0( PORTI, 1)          
#define PI2_0                B_0( PORTI, 2)          
#define PI3_0                B_0( PORTI, 3)          
#define PI4_0                B_0( PORTI, 4)          
#define PI5_0                B_0( PORTI, 5)          
#define PI6_0                B_0( PORTI, 6)          
#define PI7_0                B_0( PORTI, 7)          
#define PIU_0                PORTI&= 0B00001111
#define PIL_0                PORTI&= 0B11110000
#define PIA_0                PORTI=  0

#define PI0_R                B_R( PINI, 0)          
#define PI1_R                B_R( PINI, 1)          
#define PI2_R                B_R( PINI, 2)          
#define PI3_R                B_R( PINI, 3)          
#define PI4_R                B_R( PINI, 4)          
#define PI5_R                B_R( PINI, 5)          
#define PI6_R                B_R( PINI, 6)          
#define PI7_R                B_R( PINI, 7)          
#define PIU_R                ( PINI & 0B11110000) >> 4
#define PIL_R                PINI & 0B00001111 
#define PIA_R                PINI 

#define PI0_W( X)            X ? B_1( PORTI, 0) : B_0( PORTI, 0)          
#define PI1_W( X)            X ? B_1( PORTI, 1) : B_0( PORTI, 1)          
#define PI2_W( X)            X ? B_1( PORTI, 2) : B_0( PORTI, 2)          
#define PI3_W( X)            X ? B_1( PORTI, 3) : B_0( PORTI, 3)          
#define PI4_W( X)            X ? B_1( PORTI, 4) : B_0( PORTI, 4)          
#define PI5_W( X)            X ? B_1( PORTI, 5) : B_0( PORTI, 5)          
#define PI6_W( X)            X ? B_1( PORTI, 6) : B_0( PORTI, 6)          
#define PI7_W( X)            X ? B_1( PORTI, 7) : B_0( PORTI, 7)          
#define PIU_W( X)            PORTI= ( PORTI & 0B00001111) | ( ( X << 4) & 0B11110000)
#define PIL_W( X)            PORTI= ( PORTI & 0B11110000) | ( X & 0B00001111)
#define PIA_W( X)            PORTI= X          

// Port J

#define PJ0_OUTPUT           B_1( DDRJ, 0)          
#define PJ1_OUTPUT           B_1( DDRJ, 1)
#define PJ2_OUTPUT           B_1( DDRJ, 2)
#define PJ3_OUTPUT           B_1( DDRJ, 3)
#define PJ4_OUTPUT           B_1( DDRJ, 4)
#define PJ5_OUTPUT           B_1( DDRJ, 5)
#define PJ6_OUTPUT           B_1( DDRJ, 6)
#define PJ7_OUTPUT           B_1( DDRJ, 7)
#define PJU_OUTPUT           DDRJ|= 0B11110000
#define PJL_OUTPUT           DDRJ|= 0B00001111
#define PJA_OUTPUT           DDRJ=  0B11111111

#define PJ0_INPUT            B_0( DDRJ, 0)          
#define PJ1_INPUT            B_0( DDRJ, 1)
#define PJ2_INPUT            B_0( DDRJ, 2)
#define PJ3_INPUT            B_0( DDRJ, 3)
#define PJ4_INPUT            B_0( DDRJ, 4)
#define PJ5_INPUT            B_0( DDRJ, 5)
#define PJ6_INPUT            B_0( DDRJ, 6)
#define PJ7_INPUT            B_0( DDRJ, 7)
#define PJU_INPUT            DDRJ&= 0B00001111
#define PJL_INPUT            DDRJ&= 0B11110000
#define PJA_INPUT            DDRJ=  0

#define PJ0_PULLUP           B_1( PORTJ, 0)          
#define PJ1_PULLUP           B_1( PORTJ, 1)          
#define PJ2_PULLUP           B_1( PORTJ, 2)          
#define PJ3_PULLUP           B_1( PORTJ, 3)          
#define PJ4_PULLUP           B_1( PORTJ, 4)          
#define PJ5_PULLUP           B_1( PORTJ, 5)          
#define PJ6_PULLUP           B_1( PORTJ, 6)          
#define PJ7_PULLUP           B_1( PORTJ, 7)          
#define PJU_PULLUP           PORTJ|= 0B11110000
#define PJL_PULLUP           PORTJ|= 0B00001111
#define PJA_PULLUP           PORTJ=  0B11111111

#define PJ0_1                B_1( PORTJ, 0)          
#define PJ1_1                B_1( PORTJ, 1)          
#define PJ2_1                B_1( PORTJ, 2)          
#define PJ3_1                B_1( PORTJ, 3)          
#define PJ4_1                B_1( PORTJ, 4)          
#define PJ5_1                B_1( PORTJ, 5)          
#define PJ6_1                B_1( PORTJ, 6)          
#define PJ7_1                B_1( PORTJ, 7)          
#define PJU_1                PORTJ|= 0B11110000
#define PJL_1                PORTJ|= 0B00001111
#define PJA_1                PORTJ=  0B11111111

#define PJ0_0                B_0( PORTJ, 0)          
#define PJ1_0                B_0( PORTJ, 1)          
#define PJ2_0                B_0( PORTJ, 2)          
#define PJ3_0                B_0( PORTJ, 3)          
#define PJ4_0                B_0( PORTJ, 4)          
#define PJ5_0                B_0( PORTJ, 5)          
#define PJ6_0                B_0( PORTJ, 6)          
#define PJ7_0                B_0( PORTJ, 7)          
#define PJU_0                PORTJ&= 0B00001111
#define PJL_0                PORTJ&= 0B11110000
#define PJA_0                PORTJ=  0

#define PJ0_R                B_R( PINJ, 0)          
#define PJ1_R                B_R( PINJ, 1)          
#define PJ2_R                B_R( PINJ, 2)          
#define PJ3_R                B_R( PINJ, 3)          
#define PJ4_R                B_R( PINJ, 4)          
#define PJ5_R                B_R( PINJ, 5)          
#define PJ6_R                B_R( PINJ, 6)          
#define PJ7_R                B_R( PINJ, 7)          
#define PJU_R                ( PINJ & 0B11110000) >> 4
#define PJL_R                PINJ & 0B00001111 
#define PJA_R                PINJ 

#define PJ0_W( X)            X ? B_1( PORTJ, 0) : B_0( PORTJ, 0)          
#define PJ1_W( X)            X ? B_1( PORTJ, 1) : B_0( PORTJ, 1)          
#define PJ2_W( X)            X ? B_1( PORTJ, 2) : B_0( PORTJ, 2)          
#define PJ3_W( X)            X ? B_1( PORTJ, 3) : B_0( PORTJ, 3)          
#define PJ4_W( X)            X ? B_1( PORTJ, 4) : B_0( PORTJ, 4)          
#define PJ5_W( X)            X ? B_1( PORTJ, 5) : B_0( PORTJ, 5)          
#define PJ6_W( X)            X ? B_1( PORTJ, 6) : B_0( PORTJ, 6)          
#define PJ7_W( X)            X ? B_1( PORTJ, 7) : B_0( PORTJ, 7)          
#define PJU_W( X)            PORTJ= ( PORTJ & 0B00001111) | ( ( X << 4) & 0B11110000)
#define PJL_W( X)            PORTJ= ( PORTJ & 0B11110000) | ( X & 0B00001111)
#define PJA_W( X)            PORTJ= X          

// Port K

#define PK0_OUTPUT           B_1( DDRK, 0)          
#define PK1_OUTPUT           B_1( DDRK, 1)
#define PK2_OUTPUT           B_1( DDRK, 2)
#define PK3_OUTPUT           B_1( DDRK, 3)
#define PK4_OUTPUT           B_1( DDRK, 4)
#define PK5_OUTPUT           B_1( DDRK, 5)
#define PK6_OUTPUT           B_1( DDRK, 6)
#define PK7_OUTPUT           B_1( DDRK, 7)
#define PKU_OUTPUT           DDRK|= 0B11110000
#define PKL_OUTPUT           DDRK|= 0B00001111
#define PKA_OUTPUT           DDRK=  0B11111111

#define PK0_INPUT            B_0( DDRK, 0)          
#define PK1_INPUT            B_0( DDRK, 1)
#define PK2_INPUT            B_0( DDRK, 2)
#define PK3_INPUT            B_0( DDRK, 3)
#define PK4_INPUT            B_0( DDRK, 4)
#define PK5_INPUT            B_0( DDRK, 5)
#define PK6_INPUT            B_0( DDRK, 6)
#define PK7_INPUT            B_0( DDRK, 7)
#define PKU_INPUT            DDRK&= 0B00001111
#define PKL_INPUT            DDRK&= 0B11110000
#define PKA_INPUT            DDRK=  0

#define PK0_PULLUP           B_1( PORTK, 0)          
#define PK1_PULLUP           B_1( PORTK, 1)          
#define PK2_PULLUP           B_1( PORTK, 2)          
#define PK3_PULLUP           B_1( PORTK, 3)          
#define PK4_PULLUP           B_1( PORTK, 4)          
#define PK5_PULLUP           B_1( PORTK, 5)          
#define PK6_PULLUP           B_1( PORTK, 6)          
#define PK7_PULLUP           B_1( PORTK, 7)          
#define PKU_PULLUP           PORTK|= 0B11110000
#define PKL_PULLUP           PORTK|= 0B00001111
#define PKA_PULLUP           PORTK=  0B11111111

#define PK0_1                B_1( PORTK, 0)          
#define PK1_1                B_1( PORTK, 1)          
#define PK2_1                B_1( PORTK, 2)          
#define PK3_1                B_1( PORTK, 3)          
#define PK4_1                B_1( PORTK, 4)          
#define PK5_1                B_1( PORTK, 5)          
#define PK6_1                B_1( PORTK, 6)          
#define PK7_1                B_1( PORTK, 7)          
#define PKU_1                PORTK|= 0B11110000
#define PKL_1                PORTK|= 0B00001111
#define PKA_1                PORTK=  0B11111111

#define PK0_0                B_0( PORTK, 0)          
#define PK1_0                B_0( PORTK, 1)          
#define PK2_0                B_0( PORTK, 2)          
#define PK3_0                B_0( PORTK, 3)          
#define PK4_0                B_0( PORTK, 4)          
#define PK5_0                B_0( PORTK, 5)          
#define PK6_0                B_0( PORTK, 6)          
#define PK7_0                B_0( PORTK, 7)          
#define PKU_0                PORTK&= 0B00001111
#define PKL_0                PORTK&= 0B11110000
#define PKA_0                PORTK=  0

#define PK0_R                B_R( PINK, 0)          
#define PK1_R                B_R( PINK, 1)          
#define PK2_R                B_R( PINK, 2)          
#define PK3_R                B_R( PINK, 3)          
#define PK4_R                B_R( PINK, 4)          
#define PK5_R                B_R( PINK, 5)          
#define PK6_R                B_R( PINK, 6)          
#define PK7_R                B_R( PINK, 7)          
#define PKU_R                ( PINK & 0B11110000) >> 4
#define PKL_R                PINK & 0B00001111 
#define PKA_R                PINK 

#define PK0_W( X)            X ? B_1( PORTK, 0) : B_0( PORTK, 0)          
#define PK1_W( X)            X ? B_1( PORTK, 1) : B_0( PORTK, 1)          
#define PK2_W( X)            X ? B_1( PORTK, 2) : B_0( PORTK, 2)          
#define PK3_W( X)            X ? B_1( PORTK, 3) : B_0( PORTK, 3)          
#define PK4_W( X)            X ? B_1( PORTK, 4) : B_0( PORTK, 4)          
#define PK5_W( X)            X ? B_1( PORTK, 5) : B_0( PORTK, 5)          
#define PK6_W( X)            X ? B_1( PORTK, 6) : B_0( PORTK, 6)          
#define PK7_W( X)            X ? B_1( PORTK, 7) : B_0( PORTK, 7)          
#define PKU_W( X)            PORTK= ( PORTK & 0B00001111) | ( ( X << 4) & 0B11110000)
#define PKL_W( X)            PORTK= ( PORTK & 0B11110000) | ( X & 0B00001111)
#define PKA_W( X)            PORTK= X          


#endif