/*******************************************************************************
*   Author       -  Kiraly Tibor
*                   http://www.tkiraaly.hu
*   Date         -  2016.03.20.
*   Chip         -  Atmel AMega8
*   Compiler     -  avr-gcc ( WinAVR)
*
*   kpin - AVR port/pin kezelo fuggvenyek ( lasd: tkiraaly_kpin.h)
*   
*******************************************************************************/

#ifndef tkiraaly_kpin
#define tkiraaly_kpin




#include "tkiraaly_kpin.h"
#include <util/delay.h>




U8 kpin_bit( U8 p)                               // bit minta meghatarozasa
{
   static U8 b[]=
   {
      0B00000001,
      0B00000010,
      0B00000100,
      0B00001000,
      0B00010000,
      0B00100001,
      0B01000000,
      0B10000000,
      0B11110000,
      0B00001111,
      0B11111111
   };
   return b[ p & 0x0F];
}




void kpin_output( U8 p)                          // port lab(ak) kimenetre allitas
{
   U8 _bit;
   _bit= kpin_bit( p);
   switch( p & 0xF0)
   {
   #ifdef PORTA                                  // csak letezo portokhoz
      case 0x00: DDRA|= _bit; break;  
   #endif         
   #ifdef PORTB
      case 0x10: DDRB|= _bit; break;  
   #endif         
   #ifdef PORTC
      case 0x20: DDRC|= _bit; break;  
   #endif         
   #ifdef PORTD
      case 0x30: DDRD|= _bit; break;  
   #endif         
   #ifdef PORTE
      case 0x40: DDRE|= _bit; break;  
   #endif         
   #ifdef PORTF
      case 0x50: DDRF|= _bit; break;  
   #endif         
   #ifdef PORTG
      case 0x60: DDRG|= _bit; break;  
   #endif         
   #ifdef PORTH
      case 0x70: DDRH|= _bit; break;  
   #endif         
   #ifdef PORTI
      case 0x80: DDRI|= _bit; break;  
   #endif         
   #ifdef PORTJ
      case 0x90: DDRJ|= _bit; break;  
   #endif         
   #ifdef PORTK
      case 0xA0: DDRK|= _bit; break;  
   #endif         
   }
}


  

void kpin_input( U8 p)                           // port lab(ak) bemenetre allitas
{
   U8 _bit;
   _bit= ~ kpin_bit( p);

   switch( p & 0xF0)
   {
   #ifdef PORTA                                  // csak letezo portokhoz
      case 0x00: DDRA&= _bit; break;
   #endif
   #ifdef PORTB
      case 0x10: DDRB&= _bit; break;
   #endif
   #ifdef PORTC
      case 0x20: DDRC&= _bit; break;
   #endif
   #ifdef PORTD
      case 0x30: DDRD&= _bit; break;
   #endif
   #ifdef PORTE
      case 0x40: DDRE&= _bit; break;
   #endif
   #ifdef PORTF
      case 0x50: DDRF&= _bit; break;
   #endif
   #ifdef PORTG
      case 0x60: DDRG&= _bit; break;
   #endif
   #ifdef PORTH
      case 0x70: DDRH&= _bit; break;
   #endif
   #ifdef PORTI
      case 0x80: DDRI&= _bit; break;
   #endif
   #ifdef PORTJ
      case 0x90: DDRJ&= _bit; break;
   #endif
   #ifdef PORTK
      case 0xA0: DDRK&= _bit; break;
   #endif
   }
}




void kpin_pullup( U8 p)                          // port lab(ak) bemenetre allitas es felhuzo ellenallas bekapcsolasa
{
   kpin_input( p);
   kpin_1( p); 
}




void kpin_1( U8 p)                               // port lab(ak) 1-be allitas
{
   U8 _bit;
   _bit= kpin_bit( p);
   switch( p & 0xF0)
   {
   #ifdef PORTA                                  // csak letezo portokhoz
      case 0x00: PORTA|= _bit; break;  
   #endif         
   #ifdef PORTB
      case 0x10: PORTB|= _bit; break;  
   #endif         
   #ifdef PORTC
      case 0x20: PORTC|= _bit; break;  
   #endif         
   #ifdef PORTD
      case 0x30: PORTD|= _bit; break;  
   #endif         
   #ifdef PORTE
      case 0x40: PORTE|= _bit; break;  
   #endif         
   #ifdef PORTF
      case 0x50: PORTF|= _bit; break;  
   #endif         
   #ifdef PORTG
      case 0x60: PORTG|= _bit; break;  
   #endif         
   #ifdef PORTH
      case 0x70: PORTH|= _bit; break;  
   #endif         
   #ifdef PORTI
      case 0x80: PORTI|= _bit; break;  
   #endif         
   #ifdef PORTJ
      case 0x90: PORTJ|= _bit; break;  
   #endif         
   #ifdef PORTK
      case 0xA0: PORTK|= _bit; break;  
   #endif         
   }
}




void kpin_0( U8 p)                               // port lab(ak) 0-be allitas
{
   U8 _bit;
   _bit= ~ kpin_bit( p);
   switch( p & 0xF0)
   {
   #ifdef PORTA                                  // csak letezo portokhoz
      case 0x00: PORTA&= _bit; break;  
   #endif         
   #ifdef PORTB
      case 0x10: PORTB&= _bit; break;  
   #endif         
   #ifdef PORTC
      case 0x20: PORTC&= _bit; break;  
   #endif         
   #ifdef PORTD
      case 0x30: PORTD&= _bit; break;  
   #endif                
   #ifdef PORTE
      case 0x40: PORTE&= _bit; break;  
   #endif         
   #ifdef PORTF
      case 0x50: PORTF&= _bit; break;  
   #endif         
   #ifdef PORTG
      case 0x60: PORTG&= _bit; break;  
   #endif         
   #ifdef PORTH
      case 0x70: PORTH&= _bit; break;  
   #endif         
   #ifdef PORTI
      case 0x80: PORTI&= _bit; break;  
   #endif         
   #ifdef PORTJ
      case 0x90: PORTJ&= _bit; break;  
   #endif         
   #ifdef PORTK
      case 0xA0: PORTK&= _bit; break;  
   #endif         
   }
}




U8 kpin_rd( U8 p)                                // port lab(ak) olvasasa
{
   U8 a= 0;
   U8 _bit;
   _bit= kpin_bit( p);
   switch( p & 0xF0)
   {
   #ifdef PORTA                                  // csak letezo portokhoz
      case 0x00: a= PINA; break;
   #endif
   #ifdef PORTB
      case 0x10: a= PINB; break;
   #endif
   #ifdef PORTC
      case 0x20: a= PINC; break;
   #endif
   #ifdef PORTD
      case 0x30: a= PIND; break;
   #endif
   #ifdef PORTE
      case 0x40: a= PINE; break;
   #endif
   #ifdef PORTF
      case 0x50: a= PINF; break;
   #endif
   #ifdef PORTG
      case 0x60: a= PING; break;
   #endif
   #ifdef PORTH
      case 0x70: a= PINH; break;
   #endif
   #ifdef PORTI
      case 0x80: a= PINI; break;
   #endif
   #ifdef PORTJ
      case 0x90: a= PINJ; break;
   #endif
   #ifdef PORTK
      case 0xA0: a= PINK; break;
   #endif
   }
   a&= _bit;                                     // maszkolas
                                                 // ha nem fel vagy teljes byte
   a= ( _bit != 0xFF && _bit != 0xFF && _bit != 0xFF && a) ? 1 : 0;
   if ( _bit == 0xF0) a>>= 4;                    // felso fel byte eseten leptetes also fel byte-ba
   return a;
}




void kpin_wr( U8 p, U8 data)                     // port lab(ak) irasa
{
   U8 _bit;
   if ( ( p & 0x0F) < 8)                         // bit-ek eseten
   {
      if( data) kpin_1( p);
      else      kpin_0( p);
   }
   else                                          // byte es fel byte eseten
   {                                             
      _bit= kpin_bit( p);
      if ( _bit == 0xF0) data<<= 4;              // felso fel byte eseten leptetes fels fel byte-ba
      data&= _bit;                               // adat maszkolasa
      switch( p & 0xF0)
      {
      #ifdef PORTA                               // csak letezo portokhoz 
         case 0x00: PORTA= ( PORTA & ( ~_bit)) | data;
      #endif
      #ifdef PORTA
         case 0x10: PORTB= ( PORTB & ( ~_bit)) | data;
      #endif
      #ifdef PORTA
         case 0x20: PORTC= ( PORTC & ( ~_bit)) | data;
      #endif
      #ifdef PORTA
         case 0x30: PORTD= ( PORTD & ( ~_bit)) | data;
      #endif
      #ifdef PORTA
         case 0x40: PORTE= ( PORTE & ( ~_bit)) | data;
      #endif
      #ifdef PORTA
         case 0x50: PORTF= ( PORTF & ( ~_bit)) | data;
      #endif
      #ifdef PORTA
         case 0x60: PORTG= ( PORTG & ( ~_bit)) | data;
      #endif
      #ifdef PORTA
         case 0x70: PORTH= ( PORTH & ( ~_bit)) | data;
      #endif
      #ifdef PORTA
         case 0x80: PORTI= ( PORTI & ( ~_bit)) | data;
      #endif
      #ifdef PORTA
         case 0x90: PORTJ= ( PORTJ & ( ~_bit)) | data;
      #endif
      #ifdef PORTA
         case 0xA0: PORTK= ( PORTK & ( ~_bit)) | data;
      #endif
      }
   }
}


#endif