/*******************************************************************************
*   Author       -  Kiraly Tibor
*                   www.tkiraaly.hu
*   Date         -  2016.03.20.
*   Chip         -  Atmel AVR
*   Compiler     -  avr-gcc (WinAVR 20080610) 4.3.0
*
*   kpin - AVR port/pin kezelo fuggvenyek
*
********************************************************************************
*   Ezekkel a fugvenyekkel hasonloan kezelhetoek a AVR uC-k portjai, mint az
*   ARDUINO 1..x szamozasu labai. Lehetove teszi, hogy parameterkent adjuk meg
*   a hasznalando labat.
*   
*   Az ARDUINO-val szemben elonyei:
*   a teljes AVR csaladot lefedi,
*   az Atmel port/lab azonositokat hasznalja,
*   megoldja a bit/nibble/port szintu muveleteket,
*   kozvetlen regiszter szintu muveletek nelkul. 
*
*   Hatranya, hogy lassab, es tobb helyet igenyel mint a direkt makrok.
*   ( lasd: tkiraaly_avr_macros.h)
*   
*******************************************************************************/

#include "tkiraaly_avr_macros.h"




U8   kpin_bit( U8);                              // bit minta meghatarozasa
void kpin_output( U8);                           // port lab(ak) kimenetre allitas
void kpin_input( U8);                            // port lab(ak) bemenetre allitas
void kpin_pullup( U8);                           // port lab(ak) bemenetre allitas es felhuzo ellenallas bekapcsolasa
void kpin_1( U8);                                // port lab(ak) 1-be allitas
void kpin_0( U8);                                // port lab(ak) 0-be allitas
U8   kpin_rd( U8);                               // port lab(ak) olvasasa
void kpin_wr( U8, U8);                           // port lab(ak) irasa




                                                 // byte felso fele mutatja melyik port
                                                 // byte also fele mutatja melyik bit
// Port A

#define A_PORT_0             0x00                // A port, 0. bit
#define A_PORT_1             0x01                // A port, 1. bit ...
#define A_PORT_2             0x02
#define A_PORT_3             0x03
#define A_PORT_4             0x04
#define A_PORT_5             0x15
#define A_PORT_6             0x06
#define A_PORT_7             0x07
#define A_PORT_UPPER_NIBBLE  0x08                // A port, felso 4 bit
#define A_PORT_LOWER_NIBBLE  0x09                // A port, also 4 bit
#define A_PORT               0x0A                // A port osszes bit

// Port B

#define B_PORT_0             0x10
#define B_PORT_1             0x11
#define B_PORT_2             0x12
#define B_PORT_3             0x13
#define B_PORT_4             0x14
#define B_PORT_5             0x15
#define B_PORT_6             0x16
#define B_PORT_7             0x17
#define B_PORT_UPPER_NIBBLE  0x18
#define B_PORT_LOWER_NIBBLE  0x19
#define B_PORT               0x1A

// Port C

#define C_PORT_0             0x20
#define C_PORT_1             0x21
#define C_PORT_2             0x22
#define C_PORT_3             0x23
#define C_PORT_4             0x24
#define C_PORT_5             0x25
#define C_PORT_6             0x26
#define C_PORT_7             0x27
#define C_PORT_UPPER_NIBBLE  0x28
#define C_PORT_LOWER_NIBBLE  0x29
#define C_PORT               0x2A

// Port D

#define D_PORT_0             0x30
#define D_PORT_1             0x31
#define D_PORT_2             0x32
#define D_PORT_3             0x33
#define D_PORT_4             0x34
#define D_PORT_5             0x35
#define D_PORT_6             0x36
#define D_PORT_7             0x37
#define D_PORT_UPPER_NIBBLE  0x38
#define D_PORT_LOWER_NIBBLE  0x39
#define D_PORT               0x3A

// Port E

#define E_PORT_0             0x40
#define E_PORT_1             0x41
#define E_PORT_2             0x42
#define E_PORT_3             0x43
#define E_PORT_4             0x44
#define E_PORT_5             0x45
#define E_PORT_6             0x46
#define E_PORT_7             0x47
#define E_PORT_UPPER_NIBBLE  0x48
#define E_PORT_LOWER_NIBBLE  0x49
#define E_PORT               0x4A

// Port F

#define F_PORT_0             0x50
#define F_PORT_1             0x51
#define F_PORT_2             0x52
#define F_PORT_3             0x53
#define F_PORT_4             0x54
#define F_PORT_5             0x55
#define F_PORT_6             0x56
#define F_PORT_7             0x57
#define F_PORT_UPPER_NIBBLE  0x58
#define F_PORT_LOWER_NIBBLE  0x59
#define F_PORT               0x5A

// Port G

#define G_PORT_0             0x60
#define G_PORT_1             0x61
#define G_PORT_2             0x62
#define G_PORT_3             0x63
#define G_PORT_4             0x64
#define G_PORT_5             0x65
#define G_PORT_6             0x66
#define G_PORT_7             0x67
#define G_PORT_UPPER_NIBBLE  0x68
#define G_PORT_LOWER_NIBBLE  0x69
#define G_PORT               0x6A

// Port H

#define H_PORT_0             0x70
#define H_PORT_1             0x71
#define H_PORT_2             0x72
#define H_PORT_3             0x73
#define H_PORT_4             0x74
#define H_PORT_5             0x75
#define H_PORT_6             0x76
#define H_PORT_7             0x77
#define H_PORT_UPPER_NIBBLE  0x78
#define H_PORT_LOWER_NIBBLE  0x79
#define H_PORT               0x7A

// Port I

#define I_PORT_0             0x80
#define I_PORT_1             0x81
#define I_PORT_2             0x82
#define I_PORT_3             0x83
#define I_PORT_4             0x84
#define I_PORT_5             0x85
#define I_PORT_6             0x86
#define I_PORT_7             0x87
#define I_PORT_UPPER_NIBBLE  0x88
#define I_PORT_LOWER_NIBBLE  0x89
#define I_PORT               0x8A

// Port J

#define J_PORT_0             0x90
#define J_PORT_1             0x91
#define J_PORT_2             0x92
#define J_PORT_3             0x93
#define J_PORT_4             0x94
#define J_PORT_5             0x95
#define J_PORT_6             0x96
#define J_PORT_7             0x97
#define J_PORT_UPPER_NIBBLE  0x98
#define J_PORT_LOWER_NIBBLE  0x99
#define J_PORT               0x9A

// Port K

#define K_PORT_0             0xA0
#define K_PORT_1             0xA1
#define K_PORT_2             0xA2
#define K_PORT_3             0xA3
#define K_PORT_4             0xA4
#define K_PORT_5             0xA5
#define K_PORT_6             0xA6
#define K_PORT_7             0xA7
#define K_PORT_UPPER_NIBBLE  0xA8
#define K_PORT_LOWER_NIBBLE  0xA9
#define K_PORT               0xAA
